from __future__ import absolute_import, division, print_function

__all__ = [
    "__title__", "__summary__", "__uri__", "__version__", "__author__",
    "__email__", "__license__", "__copyright__",
]

__title__ = "historical"
__summary__ = ("Historical tracking of AWS configuration data.")
__uri__ = "https://github.com/Netflix-Skunkworks/historical"

__version__ = "0.1.2"

__author__ = "The Historical developers"
__email__ = "security@netflix.com"

__license__ = "Apache License, Version 2.0"
__copyright__ = "Copyright 2017 {0}".format(__author__)
